

import { FuncName, smlog, Str2ArrayBuffer, Parameters, PutKV, ArrayBuffer2Str, GetKV, DelKV, MakeErrRes, MakeSuccessRes, Response,
    IteratorNew, IteValue, IteKey, IteNext, IteratorFree } from "./index"

const  GREAT_THAN   :u32 = 1;
const  LESS_THAN    :u32 = 2;
const  EQUAL        :u32 = 3;
const  NOT_EQUAL    :u32 = 4;


// User should implement the invoke method
export function invoke(txid:string):Response{
    let funcName = FuncName(txid);
    let paras = Parameters(txid);

    smlog("invoke function name is "+funcName);

    let  i :i32 = 0;
    let argsNum = paras.length;
    for(i=0; i<argsNum; i++ ){
        smlog(" the arg is "+String.UTF8.decode(paras[i],true))
    }
    if (funcName == "insert") {
        let valueRange = insert(txid, paras);
        return valueRange;
    }else if (funcName == "query") {
        let valueRange = query(txid, paras);
        return valueRange;
    }else if (funcName == "delete") {
        let valueRange = del(txid, paras);
        return valueRange;
    } else if (funcName == "queryInRange") {
        let valueRange = queryInRange(txid, paras);
        return valueRange;
    } else if (funcName == "queryInRangeMatchCondition") {
        let valueRange = queryInRangeMatchCondition(txid, paras);
        return valueRange;
    }
    smlog("the function is not known:"+funcName);

    return MakeErrRes("The function name is not known:"+funcName);
}

// 插入一个key-value
function insert(txid:string, args :Array<ArrayBuffer>):Response{
    if (args.length < 2){
        smlog("the args number is not correct: "+args.length.toString());
        return MakeErrRes("the args number is not correct: "+args.length.toString());
    }
    let key = ArrayBuffer2Str(args[0])
    let value = ArrayBuffer2Str(args[1])

    PutKV(txid,key, Str2ArrayBuffer(value));
    smlog("the key is "+key+" the value is "+value)
    return MakeSuccessRes(Str2ArrayBuffer("insert success"))
}

// 查询一个key-value
function query(txid:string, args :Array<ArrayBuffer>):Response{
    if (args.length < 1){
        smlog("the args number is not correct: "+args.length.toString());
        return MakeErrRes("the args number is not correct: "+args.length.toString());
    }
    let key = ArrayBuffer2Str(args[0]);
    let value = GetKV(txid, key);
    smlog("query finished, the value is :"+ArrayBuffer2Str(value));
    return MakeSuccessRes(value);
}

// 删除一个key-value
function del(txid:string, args :Array<ArrayBuffer>):Response{
    if (args.length < 1){
        smlog("the args number is not correct: "+args.length.toString());
        //todo the empty ArrayBuffer
        return MakeErrRes("the args number is not correct: "+args.length.toString());
    }
    let key = ArrayBuffer2Str(args[0]);
    DelKV(txid, key);

    let resStr = "delMarble finished, the key is :"+key;
    smlog(resStr);

    return MakeSuccessRes(Str2ArrayBuffer(resStr));
}

// 查询指定范围的记录,范围区域为[x,y)左闭右开
function queryInRange(txid:string, args :Array<ArrayBuffer>):Response{
    if (args.length < 2){
        smlog("the args number is not correct: "+args.length.toString());
        return MakeErrRes("the args number is not correct: "+args.length.toString());
    }
    let beginKey = ArrayBuffer2Str(args[0]);
    let endKey = ArrayBuffer2Str(args[1]);

    smlog("before iterator new, the beginkey is "+beginKey+" the end key is "+endKey);
    let itor = IteratorNew(txid, beginKey, endKey);
    if (itor == 0) {
        smlog("IteratorNew fail, itor="+ itor.toString()+ "ret=" + itor.toString())
        return MakeErrRes("IteratorNew fail, itor="+ itor.toString()+ "ret=" + itor.toString());
    }

    var strRes = "";

    while(IteNext(itor)){
        strRes += IteKey(itor);
        strRes += " ";
        let v = IteValue(itor);
        strRes += ArrayBuffer2Str(v);
        strRes += ";";
    }

    let ret : i32 = 0
    ret = IteratorFree(itor);
    if (ret != 0) {
        smlog("IteratorFree fail, itor="+ itor.toString()+ "ret=" + ret.toString())
        return MakeErrRes("IteratorFree fail, itor="+ itor.toString()+ "ret=" + ret.toString());
    }

    return MakeSuccessRes(Str2ArrayBuffer(strRes));
}

// 查询指定范围内满足条件的记录,范围区域为[x,y)左闭右开
function queryInRangeMatchCondition(txid:string, args :Array<ArrayBuffer>):Response{
    if (args.length < 2){
        smlog("the args number is not correct: "+args.length.toString());
        return MakeErrRes("the args number is not correct: "+args.length.toString());
    }
    let beginKey = ArrayBuffer2Str(args[0]);
    let endKey = ArrayBuffer2Str(args[1]);
    let judge = ArrayBuffer2Str(args[2]); // 大于|等于|小于|不等于
    let base = ArrayBuffer2Str(args[3]);  // 用于比较的数值

    smlog("before iterator new, the beginkey is "+beginKey+" the end key is "+endKey);
    let itor = IteratorNew(txid, beginKey, endKey);
    if (itor == 0) {
        smlog("IteratorNew fail, itor="+ itor.toString()+ "ret=" + itor.toString())
        return MakeErrRes("IteratorNew fail, itor="+ itor.toString()+ "ret=" + itor.toString());
    }

    var strRes = "";

    while(IteNext(itor)){
        let k = IteKey(itor);
        let v = ArrayBuffer2Str(IteValue(itor));
        var temp = "";
        switch (changeJudgeToU32(judge)) {
            case GREAT_THAN:
                if (v > base){
                    temp = k + " " + v + ";";
                }
                break
            case LESS_THAN:
                if (v < base){
                    temp = k + " " + v + ";";
                }
                break
            case EQUAL:
                if (v == base){
                    temp = k + " " + v + ";";
                }
                break
            case NOT_EQUAL:
                if (v != base){
                    temp = k + " " + v + ";";
                }
                break
            default:
                smlog("judge is not support")
        }
        strRes += temp;
    }
    let ret : i32 = 0
    ret = IteratorFree(itor);
    if (ret != 0) {
        smlog("IteratorFree fail, itor="+ itor.toString()+ "ret=" + ret.toString())
        return MakeErrRes("IteratorFree fail, itor="+ itor.toString()+ "ret=" + ret.toString());
    }

    return MakeSuccessRes(Str2ArrayBuffer(strRes));
}

function changeJudgeToU32(judge: string): u32{
    if (judge == ">"){
        return GREAT_THAN
    } else if (judge == "<"){
        return LESS_THAN
    } else if (judge == "!="){
        return NOT_EQUAL
    } else if (judge == "=="){
        return EQUAL
    }
    return 0
}

// User should implement the init method
export function init(txid:string):Response{
    let funcName = FuncName(txid);
    let paras = Parameters(txid);

    smlog("init function name is "+funcName);

    let  i :i32 = 0;
    let argsNum = paras.length;
    for(i=0; i<argsNum; i++ ){
        smlog(" the arg is "+String.UTF8.decode(paras[i],true))
    }

    if (funcName == "init") {
        return initContract(txid, paras);
    }

    smlog("the function is not known:"+funcName);

    return MakeErrRes("The function name is not known:"+funcName);
}

function initContract(txid:string, args:Array<ArrayBuffer>):Response {
    if (args.length < 2){
        smlog("the args number is not correct: "+args.length.toString())
        return MakeErrRes("the args number is not correct: "+args.length.toString());
    }
    let key = ArrayBuffer2Str(args[0]);
    let value = args[1];
    smlog("the txid is "+txid+" the key is "+key+" the value is "+ArrayBuffer2Str(value));

    PutKV(txid,key, value);
    smlog("init finished.")
    return MakeSuccessRes(Str2ArrayBuffer("init success."));
}