"use strict";
// The entry file of your WebAssembly module.
exports.__esModule = true;
exports.wasm_prepare = exports.wasm_init = exports.wasm_invoke = exports.DelKV = exports.PutKV = exports.GetKV = exports.IteratorFree = exports.IteValue = exports.IteKey = exports.IteNext = exports.IteratorNew = exports.Parameters = exports.FuncName = exports.smlogInfo = exports.smlog = exports.ArrayBuffer2Str = exports.Str2ArrayBuffer = exports.MakeSuccessRes = exports.MakeErrRes = exports.Response = void 0;
// The entry file of your WebAssembly module.
var contract_1 = require("./contract");
var ARGS_BUF_LEN = 1024 * 10;
var VALUE_BUF_LEN = 1024 * 10;
var INVOKE_OK = 0;
var DELETE_KEY_NOT_EXIST = 10;
var MAX_TXID_LEN = 128;
var MAX_FUNCTION_NAME_LEN = 128;
var SC_SUCCESS = 0x0;
var SC_FAIL = 0x1;
var Response = /** @class */ (function () {
    function Response(theValue, theErr) {
        this.value = theValue;
        this.errInfo = theErr;
    }
    return Response;
}());
exports.Response = Response;
// API for user contract
// MakeErrRes make error response, called before invoke or init return
function MakeErrRes(errInfo) {
    return new Response(new ArrayBuffer(0), errInfo);
}
exports.MakeErrRes = MakeErrRes;
// MakeSuccessRes make success response, called before invoke or init return
function MakeSuccessRes(value) {
    return new Response(value, "");
}
exports.MakeSuccessRes = MakeSuccessRes;
// Str2ArrayBuffer change string to ArrayBuffer
function Str2ArrayBuffer(str) {
    return String.UTF8.encode(str, true);
}
exports.Str2ArrayBuffer = Str2ArrayBuffer;
// ArrayBuffer2Str change ArrayBuffer to string
function ArrayBuffer2Str(ab) {
    return String.UTF8.decode(ab, true);
}
exports.ArrayBuffer2Str = ArrayBuffer2Str;
// smlog write log into native world in debug level
function smlog(str) {
    var buf = String.UTF8.encode(str, true);
    log(buf);
}
exports.smlog = smlog;
// smlog write log into native world in debug level
function smlogInfo(str) {
    var buf = String.UTF8.encode(str, true);
    logInfoNative(buf);
}
exports.smlogInfo = smlogInfo;
// FuncName get function name for this init/invoke
function FuncName(txid) {
    var txidBuf = String.UTF8.encode(txid, true);
    var funcNameBuf = new ArrayBuffer(MAX_FUNCTION_NAME_LEN);
    var bufLen = MAX_FUNCTION_NAME_LEN;
    var ret = getFuncName(txidBuf, funcNameBuf, bufLen);
    if (ret != SC_SUCCESS) {
        return "";
    }
    return String.UTF8.decode(funcNameBuf, true);
}
exports.FuncName = FuncName;
// Parameters get parameters for this init/invoke
function Parameters(txid) {
    var txidAb = String.UTF8.encode(txid, true);
    var argsAb = new ArrayBuffer(ARGS_BUF_LEN);
    var bufLen = ARGS_BUF_LEN;
    var res = getArgs(txidAb, argsAb, bufLen);
    if (res != SC_SUCCESS) {
        return new Array(0);
    }
    return parseLVFromAb(argsAb);
}
exports.Parameters = Parameters;
// IteratorNew new a iterator for range query, the return value is the pointer of iterator in native world
function IteratorNew(txid, beginKey, endKey) {
    return iteratorNew(Str2ArrayBuffer(txid), Str2ArrayBuffer(beginKey), Str2ArrayBuffer(endKey));
}
exports.IteratorNew = IteratorNew;
// IteHasNext judge whether there is next value of not
// input itor: the iterator pointer
function IteNext(itor) {
    return iteNext(itor) == SC_SUCCESS;
}
exports.IteNext = IteNext;
// IteKey get the current key
// input itor: the iterator pointer
function IteKey(itor) {
    var keyBuf = new ArrayBuffer(VALUE_BUF_LEN);
    var ret = iteKey(itor, keyBuf, VALUE_BUF_LEN);
    if (ret != SC_SUCCESS) {
        return "";
    }
    return ArrayBuffer2Str(keyBuf);
}
exports.IteKey = IteKey;
// IteValue get the current value
// input itor: the iterator pointer
function IteValue(itor) {
    var valueBuf = new ArrayBuffer(VALUE_BUF_LEN);
    var ret = iteValue(itor, valueBuf, VALUE_BUF_LEN);
    if (ret != SC_SUCCESS) {
        return new ArrayBuffer(0);
    }
    var valueArr = parseLVFromAb(valueBuf);
    if (valueArr.length <= 0) {
        log(String.UTF8.encode("iteValue value Arr length is wrong, less than 0", true));
        return new ArrayBuffer(0);
    }
    return valueArr[0];
}
exports.IteValue = IteValue;
// IteratorFree release the iterator pointer
function IteratorFree(itor) {
    return iteratorFree(itor);
}
exports.IteratorFree = IteratorFree;
// GetKV get value by txid and key
function GetKV(txid, key) {
    var valueBuf = new ArrayBuffer(VALUE_BUF_LEN);
    var res = getKV(Str2ArrayBuffer(txid), Str2ArrayBuffer(key), valueBuf, VALUE_BUF_LEN);
    if (res != SC_SUCCESS) {
        log(String.UTF8.encode("Get kv failed, the error number is : " + res.toString(), true));
        return new ArrayBuffer(0);
    }
    // return valueBuf;
    var valueArr = parseLVFromAb(valueBuf);
    if (valueArr.length <= 0) {
        log(String.UTF8.encode("Get kv value Arr length is wrong, less than 0", true));
        return new ArrayBuffer(0);
    }
    return valueArr[0];
}
exports.GetKV = GetKV;
// PutKV put key and value
function PutKV(txid, key, value) {
    var res = putKV(Str2ArrayBuffer(txid), Str2ArrayBuffer(key), value, value.byteLength);
    if (res != SC_SUCCESS) {
        log(Str2ArrayBuffer("Call putKV failed, the error number is :" + res.toString()));
        return;
    }
    return;
}
exports.PutKV = PutKV;
// DelKV delete key and value
function DelKV(txid, key) {
    var res = delKV(Str2ArrayBuffer(txid), Str2ArrayBuffer(key));
    if (res != SC_SUCCESS) {
        log(Str2ArrayBuffer("Call delKV failed, the error number is :" + res.toString()));
        return SC_FAIL;
    }
    return SC_SUCCESS;
}
exports.DelKV = DelKV;
function parseLVFromAb(ab) {
    var array = new Array();
    var dv = new DataView(ab);
    var totalLen = ab.byteLength;
    var point = 0;
    while (true) {
        if (point >= totalLen - 1) {
            break;
        }
        var len = dv.getUint16(point, true);
        if (len <= 0) {
            log(String.UTF8.encode("get args end", true));
            break;
        }
        var value = ab.slice(point + 2, point + 2 + len);
        array.push(value);
        point = point + 2 + len;
    }
    return array;
}
// this function called by native world
function wasm_invoke(buffer_offset, size) {
    var txIDBuf = new ArrayBuffer(MAX_TXID_LEN);
    var ret = getTxID(txIDBuf, MAX_TXID_LEN);
    if (ret != SC_SUCCESS) {
        return;
    }
    var txIDStr = ArrayBuffer2Str(txIDBuf);
    var res = (0, contract_1.invoke)(txIDStr);
    putResult(String.UTF8.encode(txIDStr, true), res.value, res.value.byteLength, Str2ArrayBuffer(res.errInfo));
    return;
}
exports.wasm_invoke = wasm_invoke;
// this function called by native world
function wasm_init(buffer_offset, size) {
    var txIDBuf = new ArrayBuffer(MAX_TXID_LEN);
    var ret = getTxID(txIDBuf, MAX_TXID_LEN);
    if (ret != SC_SUCCESS) {
        return;
    }
    var txIDStr = ArrayBuffer2Str(txIDBuf);
    var res = (0, contract_1.init)(txIDStr);
    smlogInfo("after init ,the txid is " + txIDStr);
    putResult(String.UTF8.encode(txIDStr, true), res.value, res.value.byteLength, Str2ArrayBuffer(res.errInfo));
    return;
}
exports.wasm_init = wasm_init;
// for AssemblyScript contract, this function should be empty
function wasm_prepare() {
    return;
}
exports.wasm_prepare = wasm_prepare;
