// The entry file of your WebAssembly module.


// The entry file of your WebAssembly module.


import { invoke, init } from "./contract";

const ARGS_BUF_LEN : i32 = 1024*10;
const VALUE_BUF_LEN :i32 = 1024*10;

const INVOKE_OK : i32 = 0;
const DELETE_KEY_NOT_EXIST : i32 = 10;

const MAX_TXID_LEN :i32 = 128;
const MAX_FUNCTION_NAME_LEN = 128;

const SC_SUCCESS = 0x0;
const SC_FAIL = 0x1;


// the exported function from native world
// @ts-ignore
@external("env","logInfoNative")
declare function logInfoNative(a:ArrayBuffer):void;

// the exported function from native world
// @ts-ignore
@external("env","teeLog")
declare function log(a:ArrayBuffer):void;

// @ts-ignore
@external("env", "getTxID")
declare function getTxID(txID:ArrayBuffer, bufLen:i32):i32;


// @ts-ignore
@external("env","getFuncName")
declare function getFuncName(txid :ArrayBuffer, functionName:ArrayBuffer, bufLen:i32 ):i32;

// parse the tlv contrustor for args from ArrayBuffer
// @ts-ignore
@external("env","getArgs")
declare function getArgs(txid :ArrayBuffer, args:ArrayBuffer, bufLen:i32 ):i32;

// return: putKV error number, when success, the return is not equal to 0
// @ts-ignore
@external("env","putKV")
declare function putKV(txid:ArrayBuffer, key:ArrayBuffer, value:ArrayBuffer, size:i32 ):i32;

// return: getKV error number, when success, the return is not equal to 0
// @ts-ignore
@external("env","getKV")
declare function getKV(txid:ArrayBuffer, key:ArrayBuffer, returnValue: ArrayBuffer, size:i32):i32

// return: delKV error number, when success, the return is not equal to 0
// @ts-ignore
@external("env","delKV")
declare function delKV(txid:ArrayBuffer, key:ArrayBuffer):i32

// put the result to client
// @ts-ignore
@external("env","putResult")
declare function putResult(txid:ArrayBuffer, value:ArrayBuffer, size:i32, err:ArrayBuffer):i32

// @ts-ignore
@external("env","iteratorNew")
declare function iteratorNew(txid:ArrayBuffer, beginKey:ArrayBuffer,endKey:ArrayBuffer):i64


// @ts-ignore
@external("env","iteNext")
declare function iteNext(itor:i64):i32

// @ts-ignore
@external("env","iteKey")
declare function iteKey(itor:i64, key: ArrayBuffer, keyLen:i32):i32

// @ts-ignore
@external("env","iteValue")
declare function iteValue(itor:i64, value: ArrayBuffer, valueLen:i32):i32

// @ts-ignore
@external("env","iteratorFree")
declare function iteratorFree(itor: i64):i32


export class Response {
  value: ArrayBuffer;
  errInfo: string;
  constructor(theValue: ArrayBuffer, theErr:string) {this.value = theValue; this.errInfo = theErr}
}


// API for user contract

// MakeErrRes make error response, called before invoke or init return
export function MakeErrRes(errInfo: string):Response {
  return new Response(new ArrayBuffer(0), errInfo);
}

// MakeSuccessRes make success response, called before invoke or init return
export function MakeSuccessRes(value: ArrayBuffer):Response {
  return new Response(value, "");
}

// Str2ArrayBuffer change string to ArrayBuffer
export function Str2ArrayBuffer(str:string):ArrayBuffer{
  return String.UTF8.encode(str, true);
}

// ArrayBuffer2Str change ArrayBuffer to string
export function ArrayBuffer2Str(ab:ArrayBuffer):string{
  return String.UTF8.decode(ab, true);
}

// smlog write log into native world in debug level
export function smlog(str:string):void{
  let buf = String.UTF8.encode(str, true);
  log(buf);
}

// smlog write log into native world in debug level
export function smlogInfo(str:string):void{
  let buf = String.UTF8.encode(str, true);
  logInfoNative(buf);
}


// FuncName get function name for this init/invoke
export function FuncName(txid :string ) :string {
  let txidBuf =  String.UTF8.encode(txid, true);
  let funcNameBuf = new ArrayBuffer(MAX_FUNCTION_NAME_LEN);
  let bufLen :i32 = MAX_FUNCTION_NAME_LEN;

  let ret = getFuncName(txidBuf, funcNameBuf, bufLen);
  if (ret != SC_SUCCESS) {
    return ""
  }

  return String.UTF8.decode(funcNameBuf, true);
}

// Parameters get parameters for this init/invoke
export function Parameters(txid: string):Array<ArrayBuffer> {
  let txidAb  = String.UTF8.encode(txid,true);

  let argsAb :ArrayBuffer = new ArrayBuffer(ARGS_BUF_LEN);
  let bufLen :i32 = ARGS_BUF_LEN;

  let res = getArgs(txidAb,argsAb,bufLen);
  if (res != SC_SUCCESS) {
    return new Array<ArrayBuffer>(0);
  }

  return parseLVFromAb(argsAb);
}

// IteratorNew new a iterator for range query, the return value is the pointer of iterator in native world
export  function IteratorNew(txid:string, beginKey:string, endKey:string) :i64{
  return iteratorNew(Str2ArrayBuffer(txid), Str2ArrayBuffer(beginKey), Str2ArrayBuffer(endKey));
}

// IteHasNext judge whether there is next value of not
// input itor: the iterator pointer
export function IteNext(itor : i64):boolean {
  return iteNext(itor) == SC_SUCCESS;
}

// IteKey get the current key
// input itor: the iterator pointer
export function IteKey(itor : i64):string {
  let keyBuf = new ArrayBuffer(VALUE_BUF_LEN);

  let ret = iteKey(itor, keyBuf, VALUE_BUF_LEN);
  if (ret != SC_SUCCESS){
    return ""
  }
  return ArrayBuffer2Str(keyBuf)
}

// IteValue get the current value
// input itor: the iterator pointer
export function IteValue(itor : i64):ArrayBuffer {
  let valueBuf = new ArrayBuffer(VALUE_BUF_LEN);

  let ret = iteValue(itor, valueBuf, VALUE_BUF_LEN);
  if (ret != SC_SUCCESS){
    return new ArrayBuffer(0);
  }
  let valueArr = parseLVFromAb(valueBuf);
  if (valueArr.length <= 0) {
    log(String.UTF8.encode("iteValue value Arr length is wrong, less than 0", true));
    return new ArrayBuffer(0);
  }
  return valueArr[0];
}

// IteratorFree release the iterator pointer
export function IteratorFree(itor : i64):i32 {
  return iteratorFree(itor);
}

// GetKV get value by txid and key
export function GetKV(txid:string, key:string):ArrayBuffer{
  let valueBuf = new ArrayBuffer(VALUE_BUF_LEN);
  let res = getKV(Str2ArrayBuffer(txid), Str2ArrayBuffer(key), valueBuf, VALUE_BUF_LEN);
  if (res != SC_SUCCESS){
    log(String.UTF8.encode("Get kv failed, the error number is : "+res.toString(),true));
    return new ArrayBuffer(0);
  }
  // return valueBuf;
  let valueArr = parseLVFromAb(valueBuf);
  if (valueArr.length <= 0) {
    log(String.UTF8.encode("Get kv value Arr length is wrong, less than 0", true));
    return new ArrayBuffer(0);
  }

  return valueArr[0];
}

// PutKV put key and value
export function PutKV(txid:string, key:string, value:ArrayBuffer):void{
  let res = putKV(Str2ArrayBuffer(txid), Str2ArrayBuffer(key), value, value.byteLength);
  if (res != SC_SUCCESS){
    log(Str2ArrayBuffer("Call putKV failed, the error number is :"+res.toString()));
    return;
  }

  return;
}

// DelKV delete key and value
export function DelKV(txid:string, key:string):i32{
  let res = delKV(Str2ArrayBuffer(txid), Str2ArrayBuffer(key))
  if (res != SC_SUCCESS){
    log(Str2ArrayBuffer("Call delKV failed, the error number is :"+res.toString()));
    return SC_FAIL;
  }

  return SC_SUCCESS;
}

function parseLVFromAb(ab:ArrayBuffer):Array<ArrayBuffer> {
  let array = new Array<ArrayBuffer>();
  let dv :DataView = new DataView(ab);
  let totalLen = ab.byteLength;
  let point :i32 = 0;

  while(true){
    if(point >= totalLen-1){
      break;
    }
    let len = dv.getUint16(point,true);
    if (len <= 0){
      log(String.UTF8.encode("get args end", true));
      break;
    }

    let value = ab.slice(point+2,point+2+len);

    array.push(value);
    point = point+2+len;
  }
  return array;
}


// this function called by native world
export function wasm_invoke(buffer_offset: i32, size: i32):void{
  var txIDBuf = new ArrayBuffer(MAX_TXID_LEN);

  let ret = getTxID(txIDBuf, MAX_TXID_LEN)
  if (ret != SC_SUCCESS) {
    return;
  }
  let txIDStr = ArrayBuffer2Str(txIDBuf);

  let res = invoke(txIDStr);

  putResult(String.UTF8.encode(txIDStr, true), res.value, res.value.byteLength, Str2ArrayBuffer(res.errInfo));

  return;
}



// this function called by native world
export function wasm_init(buffer_offset: i32, size: i32):void{
  var txIDBuf = new ArrayBuffer(MAX_TXID_LEN);

  let ret = getTxID(txIDBuf, MAX_TXID_LEN)
  if (ret != SC_SUCCESS) {
    return;
  }
  let txIDStr = ArrayBuffer2Str(txIDBuf);

  let res = init(txIDStr);
  smlogInfo("after init ,the txid is "+txIDStr );

  putResult(String.UTF8.encode(txIDStr, true), res.value, res.value.byteLength, Str2ArrayBuffer(res.errInfo));

  return;
}

// for AssemblyScript contract, this function should be empty
export function wasm_prepare():void{
  return;
}